@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dmaaudio.c,v 1.3 2001/05/23 00:36:17 hercules Exp $";
+ "@(#) $Id: SDL_dmaaudio.c,v 1.5 2001/07/08 09:00:06 hercules Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -39,7 +39,13 @@ static char rcsid =
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
+#ifdef OSS_USE_SOUNDCARD_H
+/* This is installed on some systems */
+#include <soundcard.h>
+#else
+/* This is recommended by OSS */
 #include <sys/soundcard.h>
+#endif
 
 #ifndef MAP_FAILED
 #define MAP_FAILED	((Uint8 *)-1)
@@ -260,7 +266,6 @@ static int DMA_ReopenAudio(_THIS, const char *audiodev, int format, int stereo,
 		SDL_SetError("Couldn't open %s: %s", audiodev, strerror(errno));
 		return(-1);
 	}
-	ioctl(audio_fd, SNDCTL_DSP_RESET, 0);
 
 	/* Calculate the final parameters for this audio specification */
 	SDL_CalculateAudioSpec(spec);
