@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sunaudio.c,v 1.2 2001/04/26 16:50:17 hercules Exp $";
+ "@(#) $Id: SDL_sunaudio.c,v 1.3 2001/08/09 13:09:47 slouken Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -117,7 +117,7 @@ static SDL_AudioDevice *Audio_CreateDevice(int devindex)
 	return this;
 }
 
-AudioBootStrap AUDIO_bootstrap = {
+AudioBootStrap SUNAUDIO_bootstrap = {
 	"audio", "UNIX /dev/audio interface",
 	Audio_Available, Audio_CreateDevice
 };
