@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_keyboard.c,v 1.4 2001/07/23 04:34:52 slouken Exp $";
+ "@(#) $Id: SDL_keyboard.c,v 1.6 2001/11/02 19:49:14 slouken Exp $";
 #endif
 
 /* General keyboard handling code for SDL */
@@ -316,6 +316,7 @@ int SDL_KeyboardInit(void)
 	keynames[SDLK_MENU] = "menu";
 	keynames[SDLK_POWER] = "power";
 	keynames[SDLK_EURO] = "euro";
+	keynames[SDLK_UNDO] = "undo";
 
 	/* Done.  Whew. */
 	return(0);
@@ -513,16 +514,16 @@ printf("The '%s' key has been %s\n", SDL_GetKeyName(keysym->sym),
 	if ( SDL_ProcessEvents[event.type] == SDL_ENABLE ) {
 		event.key.state = state;
 		event.key.keysym = *keysym;
+		/*
+		 * jk 991215 - Added
+		 */
+		if (repeatable && (SDL_KeyRepeat.delay != 0)) {
+			SDL_KeyRepeat.evt = event;
+			SDL_KeyRepeat.firsttime = 1;
+			SDL_KeyRepeat.timestamp=SDL_GetTicks();
+		}
 		if ( (SDL_EventOK == NULL) || SDL_EventOK(&event) ) {
 			posted = 1;
-			/*
-			 * jk 991215 - Added
-			 */
-			if (repeatable && (SDL_KeyRepeat.delay != 0)) {
-				SDL_KeyRepeat.evt = event;
-				SDL_KeyRepeat.firsttime = 1;
-				SDL_KeyRepeat.timestamp=SDL_GetTicks();
-			}
 			SDL_PushEvent(&event);
 		}
 	}
@@ -547,7 +548,9 @@ void SDL_CheckKeyRepeat(void)
 		} else {
 			if ( interval > (Uint32)SDL_KeyRepeat.interval ) {
 				SDL_KeyRepeat.timestamp = now;
-				SDL_PushEvent(&SDL_KeyRepeat.evt);
+				if ( (SDL_EventOK == NULL) || SDL_EventOK(&SDL_KeyRepeat.evt) ) {
+					SDL_PushEvent(&SDL_KeyRepeat.evt);
+				}
 			}
 		}
 	}
