@@ -23,4 +23,6 @@
 /* Stub until we implement threads on this platform */
 typedef int SYS_ThreadHandle;
 
+#ifndef DISABLE_THREADS
 #define DISABLE_THREADS
+#endif
