@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_blit.h,v 1.3 2001/07/07 20:20:16 hercules Exp $";
+ "@(#) $Id: SDL_blit.h,v 1.4 2001/09/04 22:28:55 slouken Exp $";
 #endif
 
 #ifndef _SDL_blit_h
@@ -71,9 +71,6 @@ typedef struct SDL_BlitMap {
 } SDL_BlitMap;
 
 
-/* Definitions for special global blit functions */
-#include "SDL_blit_A.h"
-
 /* Functions found in SDL_blit.c */
 extern int SDL_CalculateBlit(SDL_Surface *surface);
 
