@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_surface.c,v 1.2 2001/04/26 16:50:18 hercules Exp $";
+ "@(#) $Id: SDL_surface.c,v 1.3 2001/07/31 05:36:10 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -275,7 +275,7 @@ int SDL_SetAlpha (SDL_Surface *surface, Uint32 flag, Uint8 value)
  * return true if the rectangles intersect, false otherwise
  */
 static __inline__
-SDL_bool SDL_IntersectRect(SDL_Rect *A, SDL_Rect *B, SDL_Rect *intersection)
+SDL_bool SDL_IntersectRect(const SDL_Rect *A, const SDL_Rect *B, SDL_Rect *intersection)
 {
 	int Amin, Amax, Bmin, Bmax;
 
@@ -308,7 +308,7 @@ SDL_bool SDL_IntersectRect(SDL_Rect *A, SDL_Rect *B, SDL_Rect *intersection)
 /*
  * Set the clipping rectangle for a blittable surface
  */
-SDL_bool SDL_SetClipRect(SDL_Surface *surface, SDL_Rect *rect)
+SDL_bool SDL_SetClipRect(SDL_Surface *surface, const SDL_Rect *rect)
 {
 	SDL_Rect full_rect;
 
