@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_yuv.c,v 1.2 2001/04/26 16:50:18 hercules Exp $";
+ "@(#) $Id: SDL_yuv.c,v 1.3 2001/06/25 22:16:44 hercules Exp $";
 #endif
 
 /* This is the implementation of the YUV video surface support */
@@ -48,13 +48,13 @@ SDL_Overlay *SDL_CreateYUVOverlay(int w, int h, Uint32 format,
 	overlay = NULL;
 
 	/* Display directly on video surface, if possible */
-#if 0
-	if ( (display == SDL_PublicSurface) &&
-	     ((SDL_VideoSurface->format->BytesPerPixel == 2) ||
-	      (SDL_VideoSurface->format->BytesPerPixel == 4)) ) {
-		display = SDL_VideoSurface;
+	if ( getenv("SDL_VIDEO_YUV_DIRECT") ) {
+		if ( (display == SDL_PublicSurface) &&
+		     ((SDL_VideoSurface->format->BytesPerPixel == 2) ||
+		      (SDL_VideoSurface->format->BytesPerPixel == 4)) ) {
+			display = SDL_VideoSurface;
+		}
 	}
-#endif
         yuv_hwaccel = getenv("SDL_VIDEO_YUV_HWACCEL");
 	if ( ((display == SDL_VideoSurface) && video->CreateYUVOverlay) &&
 	     (!yuv_hwaccel || (atoi(yuv_hwaccel) > 0)) ) {
