@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dgavideo.c,v 1.6 2001/07/13 10:15:52 hercules Exp $";
+ "@(#) $Id: SDL_dgavideo.c,v 1.7 2001/09/30 20:19:20 slouken Exp $";
 #endif
 
 /* DGA 2.0 based SDL video driver implementation.
@@ -531,7 +531,6 @@ SDL_Surface *DGA_SetVideoMode(_THIS, SDL_Surface *current,
 			flip_address[1] = memory_base+screen_len;
 			surfaces_mem += screen_len;
 			surfaces_len -= screen_len;
-			DGA_FlipHWSurface(this, current);
 		}
 	}
 
@@ -542,6 +541,13 @@ SDL_Surface *DGA_SetVideoMode(_THIS, SDL_Surface *current,
 	}
 	DGA_InitHWSurfaces(this, current, surfaces_mem, surfaces_len);
 
+	/* Expose the back buffer as surface memory */
+	if ( current->flags & SDL_DOUBLEBUF ) {
+		this->screen = current;
+		DGA_FlipHWSurface(this, current);
+		this->screen = NULL;
+	}
+
 	/* Set the update rectangle function */
 	this->UpdateRects = DGA_DirectUpdate;
 
