@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_nullvideo.c,v 1.2 2001/07/02 00:20:29 hercules Exp $";
+ "@(#) $Id: SDL_nullvideo.c,v 1.4 2001/11/07 17:58:41 slouken Exp $";
 #endif
 
 /* Dummy SDL video driver implementation; this is just enough to make an
@@ -42,6 +42,7 @@ static char rcsid =
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "SDL.h"
 #include "SDL_error.h"
@@ -67,7 +68,6 @@ static void DUMMY_VideoQuit(_THIS);
 /* Hardware surface functions */
 static int DUMMY_AllocHWSurface(_THIS, SDL_Surface *surface);
 static int DUMMY_LockHWSurface(_THIS, SDL_Surface *surface);
-static int DUMMY_FlipHWSurface(_THIS, SDL_Surface *surface);
 static void DUMMY_UnlockHWSurface(_THIS, SDL_Surface *surface);
 static void DUMMY_FreeHWSurface(_THIS, SDL_Surface *surface);
 
@@ -223,11 +223,6 @@ static void DUMMY_UnlockHWSurface(_THIS, SDL_Surface *surface)
 	return;
 }
 
-static int DUMMY_FlipHWSurface(_THIS, SDL_Surface *surface)
-{
-	return(0);
-}
-
 static void DUMMY_UpdateRects(_THIS, int numrects, SDL_Rect *rects)
 {
 	/* do nothing. */
