@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_fbvideo.c,v 1.5 2001/07/13 10:19:51 hercules Exp $";
+ "@(#) $Id: SDL_fbvideo.c,v 1.6 2001/07/31 20:08:51 slouken Exp $";
 #endif
 
 /* Framebuffer console based SDL video driver implementation.
@@ -49,6 +49,8 @@ static char rcsid =
 #include "SDL_fbevents_c.h"
 #include "SDL_fb3dfx.h"
 #include "SDL_fbmatrox.h"
+#include "SDL_fbriva.h"
+
 
 #if defined(i386) && defined(FB_TYPE_VGA_PLANES)
 #define VGA16_FBCON_SUPPORT
@@ -505,6 +507,13 @@ static int FB_VideoInit(_THIS, SDL_PixelFormat *vformat)
 #endif
 			FB_3DfxAccel(this, finfo.accel);
 			break;
+		    case FB_ACCEL_NV3:
+		    case FB_ACCEL_NV4:
+#ifdef FBACCEL_DEBUG
+			printf("NVidia hardware accelerator!\n");
+#endif
+			FB_RivaAccel(this, finfo.accel);
+			break;
 		    default:
 #ifdef FBACCEL_DEBUG
 			printf("Unknown hardware accelerator.\n");
