@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ggivideo.c,v 1.2.2.7 2001/02/10 07:20:06 hercules Exp $";
+ "@(#) $Id: SDL_ggivideo.c,v 1.3 2001/05/10 18:10:19 hercules Exp $";
 #endif
 
 /* GGI-based SDL video driver implementation.
@@ -75,11 +75,13 @@ static void GGI_FreeHWSurface(_THIS, SDL_Surface *surface);
 static int GGI_Available(void)
 {
 	ggi_visual_t *vis;
-	
-	vis = ggiOpen(NULL);
-	if (vis != NULL) 
-	{
-		ggiClose(vis);
+
+	vis = NULL;
+	if (ggiInit() == 0) {
+		vis = ggiOpen(NULL);
+		if (vis != NULL) {
+			ggiClose(vis);
+		}
 	}
 	return (vis != NULL);
 }
