@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gsevents.c,v 1.1 2001/06/16 03:17:45 hercules Exp $";
+ "@(#) $Id: SDL_gsevents.c,v 1.2 2001/07/31 06:08:11 slouken Exp $";
 #endif
 
 /* Handle the event stream, converting console events into SDL events */
@@ -712,10 +712,12 @@ static void handle_mouse(_THIS)
 				    case 0x02: /* DX = -1 */
 					break;
 				    case 0x0F: /* DY = +1 (map button 4) */
-					button |= (1<<3);
+                                       FB_vgamousecallback(button | (1<<3),
+                                                           1, 0, 0);
 					break;
 				    case 0x01: /* DY = -1 (map button 5) */
-					button |= (1<<4);
+                                       FB_vgamousecallback(button | (1<<4),
+                                                           1, 0, 0);
 					break;
 				}
 				break;
