@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_lowvideo.h,v 1.3 2001/04/29 23:00:03 hercules Exp $";
+ "@(#) $Id: SDL_lowvideo.h,v 1.4 2001/08/09 12:21:32 slouken Exp $";
 #endif
 
 #ifndef _SDL_lowvideo_h
@@ -87,4 +87,8 @@ extern int mouse_relative;
 /* This is really from SDL_dx5audio.c */
 extern void DX5_SoundFocus(HWND window);
 
+/* DJM: This is really from SDL_sysevents.c, we need it in
+   GDL_CreateWindow as well */
+LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
+
 #endif /* SDL_lowvideo_h */
