@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysevents.c,v 1.5 2001/06/16 01:01:42 hercules Exp $";
+ "@(#) $Id: SDL_sysevents.c,v 1.9 2001/09/13 16:15:43 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -145,8 +145,10 @@ static void WIN_GetKeyboardState(void)
 #endif /* !NO_GETKEYBOARDSTATE */
 }
 
-/* The main Win32 event handler */
-static LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
+/* The main Win32 event handler
+DJM: This is no longer static as (DX5/DIB)_CreateWindow needs it
+*/
+LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
 	SDL_VideoDevice *this = current_video;
 	static int mouse_pressed = 0;
@@ -265,6 +267,13 @@ static LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPara
 				Sint16 x, y;
 				Uint8 button, state;
 
+				/* DJM:
+				   We want the SDL window to take focus so that
+				   it acts like a normal windows "component"
+				   (e.g. gains keyboard focus on a mouse click).
+				 */
+				SetFocus(SDL_Window);
+
 				/* Figure out which button to use */
 				switch (msg) {
 					case WM_LBUTTONDOWN:
@@ -325,25 +334,15 @@ static LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPara
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL: 
 			if ( SDL_VideoSurface && ! DINPUT_FULLSCREEN() ) {
-				Sint16 x, y;
-				Uint8 button = 0;
 				int move = (short)HIWORD(wParam);
-				if(move > 0)
-					button = 4;
-				else if(move < 0)
-					button = 5;
-				if(button)
-				{
-					if ( mouse_relative ) {
-					/*	RJR: March 28, 2000
-						report internal mouse position if in relative mode */
-						x = 0; y = 0;
-					} else {
-						x = (Sint16)LOWORD(lParam);
-						y = (Sint16)HIWORD(lParam);
-					}
+				if ( move ) {
+					Uint8 button;
+					if ( move > 0 )
+						button = 4;
+					else
+						button = 5;
 					posted = SDL_PrivateMouseButton(
-								SDL_PRESSED, button, x, y);
+						SDL_PRESSED, button, 0, 0);
 				}
 			}
 			return(0);
@@ -465,10 +464,11 @@ static LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPara
 		}
 		return(0);
 
+		/* DJM: Send an expose event in this case */
 		case WM_ERASEBKGND: {
-			/* Just do nothing */ ;
+			posted = SDL_PrivateExpose();
 		}
-		return(1);
+		return(0);
 
 		case WM_CLOSE: {
 			if ( (posted = SDL_PrivateQuit()) )
@@ -493,6 +493,30 @@ static LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPara
 	return(DefWindowProc(hwnd, msg, wParam, lParam));
 }
 
+/* Allow the application handle to be stored and retrieved later */
+static void *SDL_handle = NULL;
+
+void SDL_SetModuleHandle(void *handle)
+{
+	SDL_handle = handle;
+}
+void *SDL_GetModuleHandle(void)
+{
+	void *handle;
+
+	if ( SDL_handle ) {
+		handle = SDL_handle;
+	} else {
+		/* Warning:
+		   If SDL is built as a DLL, this will return a handle to
+		   the DLL, not the application, and DirectInput may fail
+		   to initialize.
+		 */
+		handle = GetModuleHandle(NULL);
+	}
+	return(handle);
+}
+
 /* This allows the SDL_WINDOWID hack */
 const char *SDL_windowid = NULL;
 
@@ -511,12 +535,10 @@ int SDL_RegisterApp(char *name, Uint32 style, void *hInst)
 	}
 
 	/* This function needs to be passed the correct process handle
-	   by the application.  The following call just returns a handle
-	   to the SDL DLL, which is useless for our purposes and causes
-	   DirectInput to fail to initialize.
+	   by the application.
 	 */
 	if ( ! hInst ) {
-		hInst = GetModuleHandle(NULL);
+		hInst = SDL_GetModuleHandle();
 	}
 
 	/* Register the application class */
@@ -539,7 +561,7 @@ int SDL_RegisterApp(char *name, Uint32 style, void *hInst)
 	class.lpszClassName	= name;
 #endif /* _WIN32_WCE */
 	class.hbrBackground	= NULL;
-	class.hInstance		= hInst ? hInst : GetModuleHandle(0);
+	class.hInstance		= hInst;
 	class.style		= style;
 #ifdef HAVE_OPENGL
 	class.style		|= CS_OWNDC;
