@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dibvideo.c,v 1.6 2001/07/18 20:04:23 hercules Exp $";
+ "@(#) $Id: SDL_dibvideo.c,v 1.8 2001/09/04 23:21:18 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -286,7 +286,9 @@ int DIB_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	if ( DIB_CreateWindow(this) < 0 ) {
 		return(-1);
 	}
+#ifndef DISABLE_AUDIO
 	DX5_SoundFocus(SDL_Window);
+#endif
 
 	/* Determine the screen depth */
 	vformat->BitsPerPixel = DIB_SussScreenDepth();
@@ -574,7 +576,10 @@ SDL_Surface *DIB_SetVideoMode(_THIS, SDL_Surface *current,
 		if (IsZoomed(SDL_Window)) style |= WS_MAXIMIZE;
 #endif
 	}
-	SetWindowLong(SDL_Window, GWL_STYLE, style);
+
+   /* DJM: Don't piss of anyone who has setup his own window */
+   if (!SDL_windowid)
+	   SetWindowLong(SDL_Window, GWL_STYLE, style);
 
 	/* Delete the old bitmap if necessary */
 	if ( screen_bmp != NULL ) {
