@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11modes.c,v 1.7 2001/07/11 20:18:52 hercules Exp $";
+ "@(#) $Id: SDL_x11modes.c,v 1.9 2001/11/04 04:18:43 slouken Exp $";
 #endif
 
 /* Utilities for getting and setting the X display mode */
@@ -39,6 +39,11 @@ static char rcsid =
 #include "SDL_x11modes_c.h"
 #include "SDL_x11image_c.h"
 
+#ifdef HAVE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif 
+
+#define MAX(a, b)	(a > b ? a : b)
 
 #ifdef XFREE86_VM
 Bool XVidMode(GetModeInfo, (Display *dpy, int scr, XF86VidModeModeInfo *info))
@@ -99,10 +104,11 @@ static void set_best_resolution(_THIS, int width, int height)
              XVidMode(GetAllModeLines, (SDL_Display,SDL_Screen,&nmodes,&modes))){
             qsort(modes, nmodes, sizeof *modes, cmpmodes);
 #ifdef XFREE86_DEBUG
-  printf("Available modes:\n");
-  for ( i = 0; i < nmodes; ++i ) {
-    printf("Mode %d: %dx%d\n", i, modes[i]->hdisplay, modes[i]->vdisplay);
-  }
+            printf("Available modes:\n");
+            for ( i = 0; i < nmodes; ++i ) {
+                printf("Mode %d: %dx%d\n", i,
+                        modes[i]->hdisplay, modes[i]->vdisplay);
+            }
 #endif
             for ( i = nmodes-1; i > 0 ; --i ) {
                 if ( (modes[i]->hdisplay >= width) &&
@@ -202,10 +208,15 @@ int X11_GetVideoModes(_THIS)
     int nmodes;
     XF86VidModeModeInfo **modes;
 #endif
-    int i;
+    int i, n;
+    int screen_w;
+    int screen_h;
 
     vm_error = -1;
     use_vidmode = 0;
+    screen_w = DisplayWidth(SDL_Display, SDL_Screen);
+    screen_h = DisplayHeight(SDL_Display, SDL_Screen);
+
 #ifdef XFREE86_VM
     /* Metro-X 4.3.0 and earlier has a broken implementation of
        XF86VidModeGetAllModeLines() - it hangs the client.
@@ -238,8 +249,8 @@ int X11_GetVideoModes(_THIS)
 #ifdef X_XF86VidModeGetDotClocks  /* Compiled under XFree86 4.0 */
                 /* Earlier X servers hang when doing vidmode */
 		if ( vm_major < 2 ) {
-#ifdef DEBUG_XF86
-  printf("Compiled under XFree86 4.0, server is XFree86 3.X\n");
+#ifdef XFREE86_DEBUG
+                    printf("Compiled under XFree86 4.0, server is XFree86 3.X\n");
 #endif
                     buggy_X11 = 1;
                 }
@@ -255,19 +266,42 @@ int X11_GetVideoModes(_THIS)
          XVidMode(GetAllModeLines, (SDL_Display, SDL_Screen,&nmodes,&modes)) ) {
 
         qsort(modes, nmodes, sizeof *modes, cmpmodes);
-        SDL_modelist = (SDL_Rect **)malloc((nmodes+1)*sizeof(SDL_Rect *));
+        SDL_modelist = (SDL_Rect **)malloc((nmodes+2)*sizeof(SDL_Rect *));
         if ( SDL_modelist ) {
+            n = 0;
             for ( i=0; i<nmodes; ++i ) {
-                SDL_modelist[i] = (SDL_Rect *)malloc(sizeof(SDL_Rect));
-                if ( SDL_modelist[i] == NULL ) {
+                int w, h;
+
+                /* Check to see if we should add the screen size (Xinerama) */
+                w = modes[i]->hdisplay;
+                h = modes[i]->vdisplay;
+                if ( (screen_w * screen_h) >= (w * h) ) {
+                    if ( (screen_w != w) || (screen_h != h) ) {
+                        SDL_modelist[n] = (SDL_Rect *)malloc(sizeof(SDL_Rect));
+                        if ( SDL_modelist[n] ) {
+                            SDL_modelist[n]->x = 0;
+                            SDL_modelist[n]->y = 0;
+                            SDL_modelist[n]->w = screen_w;
+                            SDL_modelist[n]->h = screen_h;
+                            ++n;
+                        }
+                    }
+                    screen_w = 0;
+                    screen_h = 0;
+                }
+
+                /* Add the size from the video mode list */
+                SDL_modelist[n] = (SDL_Rect *)malloc(sizeof(SDL_Rect));
+                if ( SDL_modelist[n] == NULL ) {
                     break;
                 }
-                SDL_modelist[i]->x = 0;
-                SDL_modelist[i]->y = 0;
-                SDL_modelist[i]->w = modes[i]->hdisplay;
-                SDL_modelist[i]->h = modes[i]->vdisplay;
+                SDL_modelist[n]->x = 0;
+                SDL_modelist[n]->y = 0;
+                SDL_modelist[n]->w = w;
+                SDL_modelist[n]->h = h;
+                ++n;
             }
-            SDL_modelist[i] = NULL;
+            SDL_modelist[n] = NULL;
         }
         XFree(modes);
 
@@ -321,32 +355,64 @@ int X11_GetVideoModes(_THIS)
 
     if ( SDL_modelist == NULL ) {
         SDL_modelist = (SDL_Rect **)malloc((1+1)*sizeof(SDL_Rect *));
-        i = 0;
         if ( SDL_modelist ) {
-            SDL_modelist[i] = (SDL_Rect *)malloc(sizeof(SDL_Rect));
-            if ( SDL_modelist[i] ) {
-                SDL_modelist[i]->x = 0;
-                SDL_modelist[i]->y = 0;
-                SDL_modelist[i]->w = DisplayWidth(SDL_Display, SDL_Screen);
-                SDL_modelist[i]->h = DisplayHeight(SDL_Display, SDL_Screen);
-                ++i;
+            n = 0;
+            SDL_modelist[n] = (SDL_Rect *)malloc(sizeof(SDL_Rect));
+            if ( SDL_modelist[n] ) {
+                SDL_modelist[n]->x = 0;
+                SDL_modelist[n]->y = 0;
+                SDL_modelist[n]->w = screen_w;
+                SDL_modelist[n]->h = screen_h;
+                ++n;
             }
-            SDL_modelist[i] = NULL;
+            SDL_modelist[n] = NULL;
         }
     }
 
-#ifdef DEBUG_XF86
+#ifdef XFREE86_DEBUG
     if ( use_vidmode ) {
-        fprintf(stderr, "XFree86 VidMode is enabled\n");
+        printf("XFree86 VidMode is enabled\n");
     }
     if ( SDL_modelist ) {
-        fprintf(stderr, "X11 video mode list:\n");
+        printf("X11 video mode list:\n");
         for ( i=0; SDL_modelist[i]; ++i ) {
-            fprintf(stderr, "\t%dx%d\n",
-                SDL_modelist[i]->w, SDL_modelist[i]->h);
+            printf("\t%dx%d\n", SDL_modelist[i]->w, SDL_modelist[i]->h);
         }
     }
-#endif /* DEBUG_XF86 */
+#endif /* XFREE86_DEBUG */
+
+    /* The default X/Y fullscreen offset is 0/0 */
+    xinerama_x = 0;
+    xinerama_y = 0;
+
+#ifdef HAVE_XINERAMA
+    /* Query Xinerama extention */
+    if ( XineramaQueryExtension(SDL_Display, &i, &i) &&
+         XineramaIsActive(SDL_Display) ) {
+        /* Find out which screen is the zero'th one */
+        int screens;
+        XineramaScreenInfo *xinerama;
+
+#ifdef XINERAMA_DEBUG
+        printf("X11 detected Xinerama:\n");
+#endif
+        xinerama = XineramaQueryScreens(SDL_Display, &screens);
+        for ( i = 0; i < screens; i++ ) {
+#ifdef XINERAMA_DEBUG
+            printf("xinerama %d: %dx%d+%d+%d\n",
+                xinerama[i].screen_number,
+                xinerama[i].width, xinerama[i].height,
+                xinerama[i].x_org, xinerama[i].y_org);
+#endif
+            if ( xinerama[i].screen_number == 0 ) {
+                xinerama_x = xinerama[i].x_org;
+                xinerama_y = xinerama[i].y_org;
+            }
+        }
+        XFree(xinerama);
+    }
+#endif /* HAVE_XINERAMA */
+
     return 0;
 }
 
@@ -389,14 +455,27 @@ int X11_ResizeFullScreen(_THIS)
 {
     int x, y;
     int real_w, real_h;
+    int screen_w;
+    int screen_h;
 
+    screen_w = DisplayWidth(SDL_Display, SDL_Screen);
+    screen_h = DisplayHeight(SDL_Display, SDL_Screen);
+
+    x = xinerama_x;
+    y = xinerama_y;
     if ( currently_fullscreen ) {
         /* Switch resolution and cover it with the FSwindow */
-        move_cursor_to(this, 0, 0);
+        move_cursor_to(this, x, y);
         set_best_resolution(this, current_w, current_h);
-        move_cursor_to(this, 0, 0);
+        move_cursor_to(this, x, y);
         get_real_resolution(this, &real_w, &real_h);
-        XMoveResizeWindow(SDL_Display, FSwindow, 0, 0, real_w, real_h);
+        if ( current_w > real_w ) {
+            real_w = MAX(real_w, screen_w);
+        }
+        if ( current_h > real_h ) {
+            real_h = MAX(real_h, screen_h);
+        }
+        XMoveResizeWindow(SDL_Display, FSwindow, x, y, real_w, real_h);
         move_cursor_to(this, real_w/2, real_h/2);
 
         /* Center and reparent the drawing window */
@@ -426,6 +505,8 @@ int X11_EnterFullScreen(_THIS)
     int i, nwindows;
 #endif
     int real_w, real_h;
+    int screen_w;
+    int screen_h;
 
     okay = 1;
     if ( currently_fullscreen ) {
@@ -436,7 +517,15 @@ int X11_EnterFullScreen(_THIS)
     X11_GrabInputNoLock(this, SDL_GRAB_OFF);
 
     /* Map the fullscreen window to blank the screen */
+    screen_w = DisplayWidth(SDL_Display, SDL_Screen);
+    screen_h = DisplayHeight(SDL_Display, SDL_Screen);
     get_real_resolution(this, &real_w, &real_h);
+    if ( current_w > real_w ) {
+        real_w = MAX(real_w, screen_w);
+    }
+    if ( current_h > real_h ) {
+        real_h = MAX(real_h, screen_h);
+    }
     XMoveResizeWindow(SDL_Display, FSwindow, 0, 0, real_w, real_h);
     XMapRaised(SDL_Display, FSwindow);
     X11_WaitMapped(this, FSwindow);
